/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 
//#include <regex>
#include <algorithm>
#include <src/investigation/donnees/compteurs.h>
#include <src/investigation/donnees/condition.h>
#include <src/investigation/donnees/page.h>
#include <src/investigation/donnees/site.h>

using namespace Investigation::Donnees;

void Compteurs::operator+=(const Statistiques::Resultat& resultat)
{
	switch(resultat)
	{
		case Statistiques::Resultat::VRAI:
			++_vrai;
			break;
		case Statistiques::Resultat::FAUX:
			++_faux;
			break;
		case Statistiques::Resultat::NON_EVALUE:
			++_non_evalue;
			break;
		default:
			throw;
	}
}

void CompteursSite::operator+=(const Site& site)
{
	++_nombre_requetes;
}

void CompteursSite::operator+=(const Selection<Instruction>::Clause& conditionpage)
{
	if(not conditionpage._position) // n'ajoute pas les clauses par defaut
		return;
	auto it = _conditions_page.find(conditionpage._identifiant);
	if(it==_conditions_page.end())
	{
		CompteurConditionPage compteur(conditionpage._position,conditionpage._condition);
		compteur._compteurs += conditionpage._statistiques;
		_conditions_page.insert(std::make_pair(conditionpage._identifiant,compteur));
	}
	else
	{
		it->second._compteurs += conditionpage._statistiques;
	}
}

void CompteursSite::operator+=(const Page& page)
{
	auto it = _pages.find(page._identifiant);
	if(it==_pages.end())
	{
		CompteurPage compteur(page._position,page._identifiants,page._titre);
		compteur._compteurs += page._statistiques;
		_pages.insert(std::make_pair(page._identifiant,compteur));
	}
	else
	{
		it->second._compteurs += page._statistiques;
	}
	++_nombre_pages;
	if(page._brouillon)
	{
		++_nombre_pages_brouillon;
	}
	if(not page._titre.empty())
	{
		++_nombre_pages_avec_titre;
	}
	{
		auto niveau_information=page.niveau_information();
		auto it=_distribution_niveau_information.find(niveau_information);
		if(it==_distribution_niveau_information.end())
		{
			_distribution_niveau_information[niveau_information]=0;
		}
		++(_distribution_niveau_information[niveau_information]);
	}
	_mots_clef_pages.insert(page._identifiants.cbegin(),page._identifiants.cend());
	_page_courante = IdentifiantSP(new Identifiant(page._identifiant));
}

void CompteursSite::operator+=(const Selection<Reponse>::Clause& conditionreponse)
{
	if(not conditionreponse._position) // n'ajoute pas les clauses par defaut
		return;
	if(not _page_courante)
		throw;
	auto reponses = _conditions_reponse.find(*_page_courante);
	if(reponses==_conditions_reponse.end())
	{
		reponses = (_conditions_reponse.insert(std::make_pair(*_page_courante,std::map<Identifiant,CompteurConditionReponse>()))).first;
	}
	auto it = reponses->second.find(conditionreponse._identifiant);
	if(it==reponses->second.end())
	{
		CompteurConditionReponse compteur(conditionreponse._position,conditionreponse._condition);
		compteur._compteurs += conditionreponse._statistiques;
		(reponses->second).insert(std::make_pair(conditionreponse._identifiant,compteur));
	}
	else
	{
		it->second._compteurs += conditionreponse._statistiques;
	}
}
void CompteursSite::operator+=(const std::string& motclef)
{
	_mots_clef_reponses.insert(motclef);
}

bool equivalence(const std::string& id1, const std::string& id2)
{
	std::string petit;
	std::string grand;
	if(id1.size()<id2.size())
	{
		petit = id1;
		grand = id2;
	}
	else
	{
		petit = id2;
		grand = id1;
	}
	if(petit.empty())
		return false;
	if(grand.substr(0,petit.size())==petit)
		return true;
	return false;
}
std::set<std::string> split(const std::string& str, const char delim)
{
	// remplacer par sregex_token_iterator?
	//std::regex regdelim(delim);
	//std::regex_token_iterator<std::string::const_iterator> premier(str.begin(),str.end(),regdelim,-1), dernier;
	//return std::set<std::string>(premier,dernier);
	std::set<std::string> res;
	std::string s;
	for(auto it=str.cbegin() ; it!=str.cend() ; ++it)
	{
		if(*it==delim)
		{
			res.insert(s);
			s="";
		}
		else
		{
			s+=*it;
		}
	}
	res.insert(s);
	return res;
}

size_t appariement(const std::set<std::string>& set_id1, const std::set<std::string>& set_id2)
{
	size_t erreurs=0;
	for(auto it=set_id1.cbegin() ; it!=set_id1.cend() ; ++it)
	{
		bool trouve=false;
		for(auto itit=set_id2.cbegin() ; (itit!=set_id2.cend()) and not trouve; ++itit)
		{
			if(equivalence(*it,*itit))
				trouve = true;
		}
		if(not trouve)
			++erreurs;
	}
	return erreurs;
}

bool operator%(const std::string& id1, const std::string& id2)
{
	if(equivalence(id1,id2))
		return true;
	std::set<std::string> set_id1 = split(id1,'_');
	std::set<std::string> set_id2 = split(id2,'_');
	return (appariement(set_id1,set_id2)+appariement(set_id2,set_id1))<2;
}
std::set<std::set<std::string> > CompteursSite::groupes_identifiants(const std::set<std::string>& identifiants)
{
	std::set<std::set<std::string> > groupes_identifiants;
	for(auto it=identifiants.cbegin() ; it!=identifiants.cend() ; ++it)
	{
		std::set<std::string> groupe;
		for(auto itit=identifiants.cbegin() ; itit!=identifiants.cend() ; ++itit)
		{
			if((*it)%(*itit))
				groupe.insert(*itit);
		}
		groupes_identifiants.insert(groupe);
	}
	std::set<std::set<std::string> > groupes_identifiants_epure;
	for(auto it=groupes_identifiants.cbegin() ; it!=groupes_identifiants.cend() ; ++it)
	{
		bool inclus=true;
		for(auto itit=groupes_identifiants.cbegin() ; itit!=groupes_identifiants.cend() ; ++itit)
		{
			if(it==itit)
				continue;
			if(std::includes(itit->cbegin(),itit->cend(),it->cbegin(),it->cend()))
				inclus=false;
		}
		if(inclus)
		{
			groupes_identifiants_epure.insert(*it);
		}
	}
	return groupes_identifiants_epure;
}
CompteursSite::CompteursSite()
	:_nombre_requetes(0), _nombre_pages(0), _nombre_pages_brouillon(0), _nombre_pages_avec_titre(0) { }
