/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_QUESTION__
#define __INVESTIGATION_DONNEES_QUESTION__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(Question); DECL_SHARED_PTR(Reponse); typedef std::vector<ReponseSP> SequenceReponses; } }
#include <src/investigation/donnees/page.h>
#include <src/investigation/donnees/html.h>
namespace Investigation
{
	namespace Donnees
	{
		class Reponse
		{
			public:
				virtual ~Reponse() {};
				virtual Html::SequenceLi operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const = 0;
				virtual ReponseSP clone() const = 0;
				virtual std::set<std::string> operator[] (const Investigation::Donnees::Requete& requete) = 0;
				virtual bool operator%(const Reponse& reponse) const = 0;
				virtual void liens_valides(const std::set<std::string>& identifiants) const = 0;
				virtual void propage_infos(const std::string& page, const std::string& question) = 0;
				virtual void compte(CompteursSite& compteurs) const = 0;
		};

		class Question : public ContenuPage
		{
			public:
				Question(const std::string& question, const SequenceReponses& reponses) ;
				virtual ~Question(); 
				virtual Html::SequenceContenuDiv operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const ;
				virtual ContenuPageSP clone() const ;
				virtual std::set<std::string> operator[] (const Investigation::Donnees::Requete& requete) ;
				virtual bool operator%(const ContenuPage& contenu_page) const ;
				bool operator%(const Question& question) const ;
				virtual void liens_valides(const std::set<std::string>& identifiants) const ;
				virtual void propage_infos(const std::string& page) ;
				virtual long int niveau_information() const ;
				virtual void compte(CompteursSite& compteurs) const;
			private:
				std::string _question;
				SequenceReponses _reponses;
		};

	}
}
#endif
