/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#include <src/investigation/donnees/reponsechoix.h>

#include <src/investigation/donnees/requete.h>
#include <src/investigation/donnees/liens.h>
#include <src/investigation/donnees/compteurs.h>

using namespace Investigation::Donnees;

ReponseChoix::ReponseChoix(const std::string& identifiant, const std::string& intitule, const Investigation::Global::Source::PositionSP& position)
	:_identifiant(identifiant), _intitule(intitule), _position(position) { }
	ReponseChoix::~ReponseChoix() { }
	Html::SequenceLi ReponseChoix::operator() (const std::string& methode, const Requete& requete, Liens& liens) const
{
	Html::Li item_reponse("investigation page response");
	Requete nouvelle_requete = (requete << Requete::MotClef(_identifiant,_intitule,_question,_page));
	std::string lien = liens[nouvelle_requete];
	Html::A lien_reponse(methode+lien,"investigation page response");
	lien_reponse << Html::Texte(_intitule);
	item_reponse << lien_reponse;
	Html::SequenceLi seq;
	seq.push_back(item_reponse);
	return seq;
}
ReponseSP ReponseChoix::clone() const
{
	ReponseSP reponse(new ReponseChoix(_identifiant,_intitule,_position));
	return reponse;
}
std::set<std::string> ReponseChoix::operator[] (const Requete& requete)
{
	std::set<std::string> mc;
	mc.insert(_identifiant);
	return mc;
}
bool ReponseChoix::operator%(const Reponse& reponse) const
{
	const ReponseChoix& reponsechoix=dynamic_cast<const ReponseChoix&>(reponse); // throw bad_cast
	return (*this)%(reponsechoix);
}
bool ReponseChoix::operator%(const ReponseChoix& reponsechoix) const
{
	return true;
}
void ReponseChoix::liens_valides(const std::set<std::string>& identifiants) const
{
	if(identifiants.find(_identifiant)==identifiants.end())
	{
		throw LienInvalide(_position,_identifiant);
	}
}
void ReponseChoix::propage_infos(const std::string& page, const std::string& question)
{
	_question=question;
	_page=page;
}

void ReponseChoix::compte(CompteursSite& compteurs) const
{
	compteurs += _identifiant;
}
