/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_SELECTION__
#define __INVESTIGATION_DONNEES_SELECTION__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { } }
#include <src/global/sources/position.h>
#include <src/investigation/donnees/condition.h>
#include <src/investigation/donnees/statistiques.h>
#include <src/investigation/donnees/identifiants.h>

namespace Investigation
{
	namespace Donnees
	{
		class CompteursSite;
		template<typename Contenu>
		class Selection
		{
			public:
				struct Clause
				{
					friend class Selection<Contenu>;
					friend class CompteursSite;
					Clause(const ConditionSP& condition, const std::vector<SHARED_PTR(Contenu)>& contenu, const Investigation::Global::Source::PositionSP& position)
					:_condition(condition), _contenu(contenu), _position(position) {};
					Clause(const std::vector<SHARED_PTR(Contenu)>& contenu)
					:_contenu(contenu) {};
					Clause(const ConditionSP& condition, const std::vector<SHARED_PTR(Contenu)>& contenu, const Investigation::Global::Source::PositionSP& position, const Identifiant& identifiant)
					:_condition(condition), _contenu(contenu), _position(position), _identifiant(identifiant) {};
					Clause clone() const
					{
						decltype(_contenu) contenu;
						for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
						{
							contenu.push_back((*it)->clone());
						}
						return Clause(_condition,contenu,_position,_identifiant);
					};
					std::set<std::string> operator[](const Investigation::Donnees::Requete& requete)
					{
						std::set<std::string> mc;
						for(auto it=_contenu.begin() ; it!=_contenu.end() ; ++it)
						{
							auto itmc = (**it)[requete];
							mc.insert(itmc.cbegin(),itmc.cend());
						}
						return mc;
					};
					bool operator%(const Clause& clause) const
					{
						if(_statistiques!=clause._statistiques)
							return false;
						if(_contenu.size()!=clause._contenu.size())
							throw;
						for(auto it=_contenu.cbegin(), itc=clause._contenu.cbegin() ; it!=_contenu.cend() ; ++it, ++itc)
						{
							if(not ((**it)%(**itc)))
								return false;
						}
						return true;
					};
					void compte(CompteursSite& compteurs) const;
					private:
						ConditionSP _condition;
						std::vector<SHARED_PTR(Contenu)> _contenu;
						Investigation::Global::Source::PositionSP _position;
					public:
						Statistiques _statistiques;
						Identifiant _identifiant;
				};

				typedef std::vector<Clause> SequenceClauses;
				Selection(const SequenceClauses& clauses, const std::vector<SHARED_PTR(Contenu)>& clause_par_defaut)
				:_clauses(clauses), _clause_par_defaut(clause_par_defaut) { };
				virtual ~Selection() {};
				Selection<Contenu>& operator<<(const Clause& clause)
				{
					_clauses.push_back(clause);
					return *this;
				}
				Selection<Contenu>& operator<<(const SHARED_PTR(Contenu)& clause_par_defaut)
				{
					_clause_par_defaut._contenu.push_back(clause_par_defaut);
					return *this;
				}
				std::vector<SHARED_PTR(Contenu)> operator() (const Investigation::Donnees::Requete& requete) const
				{
					for(auto it=_clauses.cbegin() ; it!=_clauses.cend() ; ++it)
					{
						if((*(it->_condition))(requete))
						{
							return it->_contenu;
						}
					}
					return _clause_par_defaut._contenu;
				};
				std::set<std::string> operator[] (const Investigation::Donnees::Requete& requete)
				{
					for(auto it=_clauses.begin() ; it!=_clauses.end() ; ++it)
					{
						if((*(it->_condition))(requete))
						{
							it->_statistiques(true);
							return (*it)[requete];
						}
						else
						{
							it->_statistiques(false);
						}

					}
					_clause_par_defaut._statistiques(true);
					return _clause_par_defaut[requete];
				};
				bool operator%(const Selection<Contenu>& selection) const
				{
					if(_clauses.size()!=selection._clauses.size())
						throw;
					for(auto it=_clauses.cbegin(), itc=selection._clauses.cbegin() ; it!=_clauses.cend() ; ++it, ++itc)
					{
						if(not ((*it)%(*itc)))
							return false;
					}
					return _clause_par_defaut%selection._clause_par_defaut;
				};
				Selection<Contenu> clone_interne() const
				{
					SequenceClauses clauses;
					for(auto it=_clauses.cbegin() ; it!=_clauses.cend() ; ++it)
					{
						clauses.push_back(it->clone());
					}
					return Selection(clauses,_clause_par_defaut.clone());
				};
				operator std::vector<SHARED_PTR(Contenu)> () const
				{
					std::vector<SHARED_PTR(Contenu)> resultat;
					for(auto it=_clauses.cbegin() ; it!=_clauses.cend() ; ++it)
					{
						resultat.insert(resultat.end(),it->_contenu.cbegin(),it->_contenu.cend());
					}
					resultat.insert(resultat.end(),_clause_par_defaut._contenu.cbegin(),_clause_par_defaut._contenu.cend());
					return resultat;
				};
				void compte(CompteursSite& compteurs) const
				{
					for(auto it=_clauses.cbegin() ; it!=_clauses.cend() ; ++it)
					{
						it->compte(compteurs);
					}
					_clause_par_defaut.compte(compteurs);
				};
			private:
				SequenceClauses _clauses;
				Clause _clause_par_defaut;
				Selection(const SequenceClauses& clauses, const Clause& clause_par_defaut)
				:_clauses(clauses), _clause_par_defaut(clause_par_defaut) { };
		};
	}
}
#endif
